from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ConversionValueDTO(BaseModel):
    currency: Optional[str] = None
    conversion_category: Optional[str] = None
    value: Optional[float] = None


ConversionValueDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.linked_term_churn_params import LinkedTermChurnParams
from pianosdk.publisher.models.linked_term_purchase_params import LinkedTermPurchaseParams
from pianosdk.publisher.models.linked_term_upgrade_params import LinkedTermUpgradeParams
from typing import Dict
from typing import List


class LinkedTermSubscriptionParams(BaseModel):
    subscription_id: Optional[str] = None
    user_token: Optional[str] = None
    external_term_id: Optional[str] = None
    state: Optional[str] = None
    upgrade: Optional['LinkedTermUpgradeParams'] = None
    valid_to: Optional[int] = None
    auto_renew: Optional[bool] = None
    purchase: Optional['LinkedTermPurchaseParams'] = None
    churn: Optional['LinkedTermChurnParams'] = None
    access_custom_data: Optional[str] = None
    period: Optional[str] = None
    user_custom_fields: Optional[Dict[str, str]] = None


LinkedTermSubscriptionParams.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class Source(BaseModel):
    type: Optional[str] = None
    source_names: Optional[List[str]] = None
    abandoned: Optional[bool] = None


Source.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.id.models.publisher_custom_field_response import PublisherCustomFieldResponse
from typing import List


class PublisherUserProfileResponse(BaseModel):
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    uid: Optional[str] = None
    email: Optional[str] = None
    create_date: Optional[int] = None
    reset_password_email_sent: Optional[bool] = None
    password: Optional[str] = None
    custom_fields: Optional['List[PublisherCustomFieldResponse]'] = None


PublisherUserProfileResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TaxSupportDTO(BaseModel):
    caption: Optional[str] = None
    tax_format: Optional[str] = None


TaxSupportDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.offer_template_sub_history import OfferTemplateSubHistory
from typing import List


class OfferTemplateHistory(BaseModel):
    history_content: Optional[str] = None
    history_comment: Optional[str] = None
    offer_template_history_event: Optional[str] = None
    offer_template_id: Optional[str] = None
    history_list: Optional['List[OfferTemplateSubHistory]'] = None


OfferTemplateHistory.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.mail import Mail


class SendEmailStep(BaseModel):
    optional: Optional[bool] = None
    enabled: Optional[bool] = None
    email: Optional['Mail'] = None


SendEmailStep.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class SwgResponse(BaseModel):
    subscription_token: Optional[str] = None
    detail: Optional[str] = None
    products: Optional[List[str]] = None


SwgResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term import Term
from typing import List


class PeriodSettings(BaseModel):
    period_id: Optional[str] = None
    title_editable: Optional[bool] = None
    sell_date_editable: Optional[bool] = None
    begin_date_editable: Optional[bool] = None
    end_date_editable: Optional[bool] = None
    period_deletable: Optional[bool] = None
    dependent_terms: Optional['List[Term]'] = None


PeriodSettings.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserSubscriptionAccount(BaseModel):
    account_id: Optional[str] = None
    user_id: Optional[str] = None
    email: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    personal_name: Optional[str] = None
    redeemed: Optional[datetime] = None


UserSubscriptionAccount.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CountryModel(BaseModel):
    country_name: Optional[str] = None
    country_code: Optional[str] = None


CountryModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class InstrumentDTO(BaseModel):
    require_3ds: Optional[bool] = None
    instrument_id: Optional[str] = None
    token: Optional[str] = None
    transaction_id: Optional[str] = None
    redirect_url: Optional[str] = None


InstrumentDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.anon.models.upgrade_offer_option import UpgradeOfferOption
from typing import List


class UpgradeOfferData(BaseModel):
    change_options: Optional['List[UpgradeOfferOption]'] = None


UpgradeOfferData.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.address_config import AddressConfig
from pianosdk.publisher.models.avalara_origin_address import AvalaraOriginAddress
from typing import List


class AvalaraConfiguration(BaseModel):
    avalara_account_id: Optional[str] = None
    avalara_license_key: Optional[str] = None
    avalara_company_code: Optional[str] = None
    avalara_sales_invoice_enabled: Optional[bool] = None
    avalara_return_invoice_enabled: Optional[bool] = None
    avalara_collect_address_enabled: Optional[bool] = None
    avalara_address_config_us: Optional['List[AddressConfig]'] = None
    avalara_address_config_ca: Optional['List[AddressConfig]'] = None
    avalara_origin_address: Optional['AvalaraOriginAddress'] = None


AvalaraConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserAddressDto(BaseModel):
    company_name: Optional[str] = None
    country_name: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    region_name: Optional[str] = None
    address1: Optional[str] = None
    address2: Optional[str] = None
    phone: Optional[str] = None
    postal_code: Optional[str] = None
    city: Optional[str] = None


UserAddressDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.language_stats import LanguageStats
from typing import List


class TranslationStats(BaseModel):
    keys_count: Optional[int] = None
    languages_count: Optional[int] = None
    languages_stats: Optional['List[LanguageStats]'] = None


TranslationStats.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LicenseeRepresentative(BaseModel):
    email: Optional[str] = None


LicenseeRepresentative.model_rebuild()



from pianosdk.anon.models.access import Access
from pianosdk.anon.models.access_dto import AccessDTO
from pianosdk.anon.models.access_token_list import AccessTokenList
from pianosdk.anon.models.billing_plan_translation_request import BillingPlanTranslationRequest
from pianosdk.anon.models.check_subscription_response import CheckSubscriptionResponse
from pianosdk.anon.models.client_configurations_dto import ClientConfigurationsDto
from pianosdk.anon.models.consent_model import ConsentModel
from pianosdk.anon.models.country import Country
from pianosdk.anon.models.currency import Currency
from pianosdk.anon.models.datatrans_apple_pay_config import DatatransApplePayConfig
from pianosdk.anon.models.external_css import ExternalCss
from pianosdk.anon.models.key_source import KeySource
from pianosdk.anon.models.light_offer_template_version import LightOfferTemplateVersion
from pianosdk.anon.models.linked_term_checkout_track_params import LinkedTermCheckoutTrackParams
from pianosdk.anon.models.linked_term_event_page_view_content import LinkedTermEventPageViewContent
from pianosdk.anon.models.linked_term_event_session import LinkedTermEventSession
from pianosdk.anon.models.linked_term_event_session_offer import LinkedTermEventSessionOffer
from pianosdk.anon.models.linked_term_page_view_content_params import LinkedTermPageViewContentParams
from pianosdk.anon.models.linked_term_session_params import LinkedTermSessionParams
from pianosdk.anon.models.locale_model import LocaleModel
from pianosdk.anon.models.o_auth_token import OAuthToken
from pianosdk.anon.models.passwordless_purchase_check_result import PasswordlessPurchaseCheckResult
from pianosdk.anon.models.passwordless_purchase_complete_result import PasswordlessPurchaseCompleteResult
from pianosdk.anon.models.performance_metrics_dto import PerformanceMetricsDto
from pianosdk.anon.models.public_captcha_config import PublicCaptchaConfig
from pianosdk.anon.models.region import Region
from pianosdk.anon.models.resource import Resource
from pianosdk.anon.models.resource_dto import ResourceDto
from pianosdk.anon.models.source import Source
from pianosdk.anon.models.swg_response import SwgResponse
from pianosdk.anon.models.template_context import TemplateContext
from pianosdk.anon.models.term import Term
from pianosdk.anon.models.term_conversion import TermConversion
from pianosdk.anon.models.upgrade_offer_data import UpgradeOfferData
from pianosdk.anon.models.upgrade_offer_option import UpgradeOfferOption
from pianosdk.anon.models.user import User
from pianosdk.anon.models.user_audit import UserAudit
from pianosdk.anon.models.user_dto import UserDto
from pianosdk.anon.models.user_info import UserInfo
from pianosdk.anon.models.user_model import UserModel
from pianosdk.anon.models.user_subscription import UserSubscription



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UpgradeOfferOption(BaseModel):
    option_id: Optional[str] = None
    subscription_from_id: Optional[str] = None
    subscription_from_billing_plan: Optional[str] = None
    subscription_from_billing_period: Optional[str] = None
    term_from_id: Optional[str] = None
    term_from_name: Optional[str] = None
    term_from_type: Optional[str] = None
    access_period_from_id: Optional[str] = None
    access_period_from_name: Optional[str] = None
    resource_from_name: Optional[str] = None
    term_to_id: Optional[str] = None
    term_to_name: Optional[str] = None
    term_to_type: Optional[str] = None
    access_period_to_id: Optional[str] = None
    access_period_to_name: Optional[str] = None
    resource_to_name: Optional[str] = None
    base_term_to_billing_plan: Optional[str] = None
    term_to_billing_period: Optional[str] = None
    billing_timing: Optional[int] = None
    immediate_access: Optional[bool] = None
    billing_date: Optional[datetime] = None
    billing_date_text: Optional[str] = None
    description: Optional[str] = None


UpgradeOfferOption.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserPaymentRefundDTO(BaseModel):
    status: Optional[str] = None
    refund_external_tx_id: Optional[str] = None
    refund_downstream_external_tx_id: Optional[str] = None


UserPaymentRefundDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class TeamMember(BaseModel):
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    personal_name: Optional[str] = None
    email: Optional[str] = None
    uid: Optional[str] = None
    create_date: Optional[datetime] = None
    last_login: Optional[datetime] = None
    invitation_expired: Optional[bool] = None
    permissions: Optional[List[str]] = None


TeamMember.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.resource import Resource
from pianosdk.publisher.models.term import Term
from pianosdk.publisher.models.user import User
from pianosdk.publisher.models.user_address import UserAddress
from pianosdk.publisher.models.user_subscription_account import UserSubscriptionAccount
from typing import List


class UserSubscription(BaseModel):
    subscription_id: Optional[str] = None
    auto_renew: Optional[bool] = None
    will_auto_renew: Optional[bool] = None
    next_bill_date: Optional[datetime] = None
    next_renewal_date: Optional[datetime] = None
    will_access_period_be_changed: Optional[bool] = None
    next_verificaition_date: Optional[datetime] = None
    payment_method: Optional[str] = None
    billing_plan: Optional[str] = None
    user_payment_info_id: Optional[str] = None
    status: Optional[str] = None
    status_name: Optional[str] = None
    status_name_in_reports: Optional[str] = None
    term: Optional['Term'] = None
    resource: Optional['Resource'] = None
    user: Optional['User'] = None
    start_date: Optional[datetime] = None
    cancelable: Optional[bool] = None
    cancelable_and_refundadle: Optional[bool] = None
    user_address: Optional['UserAddress'] = None
    psc_subscriber_number: Optional[str] = None
    external_api_name: Optional[str] = None
    conversion_result: Optional[str] = None
    is_in_trial: Optional[bool] = None
    trial_period_end_date: Optional[datetime] = None
    trial_amount: Optional[float] = None
    trial_currency: Optional[str] = None
    end_date: Optional[datetime] = None
    charge_count: Optional[int] = None
    upi_ext_customer_id: Optional[str] = None
    upi_ext_customer_id_label: Optional[str] = None
    shared_account_limit: Optional[int] = None
    can_manage_shared_subscription: Optional[bool] = None
    shared_accounts: Optional['List[UserSubscriptionAccount]'] = None
    status_display: Optional[str] = None
    is_active: Optional[bool] = None
    deferred_cancelable: Optional[bool] = None
    can_be_resumed: Optional[bool] = None
    auto_renew_change_enabled: Optional[bool] = None
    in_grace_period: Optional[bool] = None
    external_sub_id: Optional[str] = None
    access_custom_data: Optional[str] = None


UserSubscription.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class TargetTermChangeVariant(BaseModel):
    name: Optional[str] = None
    term_id: Optional[str] = None
    term_name: Optional[str] = None
    term_type: Optional[str] = None
    period_id: Optional[str] = None
    period_name: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    variant_type: Optional[str] = None
    tooltip: Optional[str] = None
    prorate_disabled: Optional[str] = None
    enabled: Optional[str] = None
    available_billing_timings: Optional[List[int]] = None


TargetTermChangeVariant.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.page_info import PageInfo
from pianosdk.publisher.models.show_option_in_counters import ShowOptionInCounters
from pianosdk.publisher.models.upgrade_option import UpgradeOption
from typing import List


class UpgradeOptions(BaseModel):
    page_info: Optional['PageInfo'] = None
    upgrade_options: Optional['List[UpgradeOption]'] = None
    show_option_in_counters: Optional['ShowOptionInCounters'] = None


UpgradeOptions.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.billing_timing_option import BillingTimingOption
from typing import List


class TargetEligibleTerm(BaseModel):
    term_id: Optional[str] = None
    term_name: Optional[str] = None
    type: Optional[str] = None
    available_billing_timings: Optional['List[BillingTimingOption]'] = None
    billing_timing: Optional['BillingTimingOption'] = None
    immediate_access: Optional[bool] = None
    variant_type: Optional[str] = None
    period_id: Optional[str] = None
    period_name: Optional[str] = None
    tooltip: Optional[str] = None
    enabled: Optional[bool] = None
    name: Optional[str] = None
    force_auto_renew: Optional[bool] = None


TargetEligibleTerm.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.term import Term


class PublisherTermGiftApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create_gift_term(self, aid: str, name: str, rid: str, vouchering_policy_redemption_url: str, term_type: str = 'subscription', description: str = None, product_category: str = None, billing_plan_period: str = None, billing_plan_price: float = None, billing_plan_currency: str = None, payment_allow_promo_codes: bool = True, schedule_id: str = None, schedule_billing_model: str = None, shared_account_count: int = None, shared_redemption_url: str = None) -> ApiResponse[Term]:
        _url_path = '/api/v3/publisher/term/gift/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'product_category': _encode_parameter(product_category),
            'rid': _encode_parameter(rid),
            'billing_plan_period': _encode_parameter(billing_plan_period),
            'billing_plan_price': _encode_parameter(billing_plan_price),
            'billing_plan_currency': _encode_parameter(billing_plan_currency),
            'payment_allow_promo_codes': _encode_parameter(payment_allow_promo_codes),
            'vouchering_policy_redemption_url': _encode_parameter(vouchering_policy_redemption_url),
            'schedule_id': _encode_parameter(schedule_id),
            'schedule_billing_model': _encode_parameter(schedule_billing_model),
            'term_type': _encode_parameter(term_type),
            'shared_account_count': _encode_parameter(shared_account_count),
            'shared_redemption_url': _encode_parameter(shared_redemption_url)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Term)
        return _result

    def update_gift_term(self, aid: str, term_id: str, name: str, rid: str, vouchering_policy_redemption_url: str, term_type: str = 'subscription', description: str = None, product_category: str = None, billing_plan_period: str = None, billing_plan_price: float = None, billing_plan_currency: str = None, payment_allow_promo_codes: bool = True, schedule_id: str = None, schedule_billing_model: str = None, shared_account_count: int = None, shared_redemption_url: str = None, collect_shipping_address: bool = None, collect_address: bool = None, delivery_zone: List[str] = None, default_country: str = None) -> ApiResponse[Term]:
        _url_path = '/api/v3/publisher/term/gift/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'term_id': _encode_parameter(term_id),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'product_category': _encode_parameter(product_category),
            'rid': _encode_parameter(rid),
            'billing_plan_period': _encode_parameter(billing_plan_period),
            'billing_plan_price': _encode_parameter(billing_plan_price),
            'billing_plan_currency': _encode_parameter(billing_plan_currency),
            'payment_allow_promo_codes': _encode_parameter(payment_allow_promo_codes),
            'vouchering_policy_redemption_url': _encode_parameter(vouchering_policy_redemption_url),
            'schedule_id': _encode_parameter(schedule_id),
            'schedule_billing_model': _encode_parameter(schedule_billing_model),
            'term_type': _encode_parameter(term_type),
            'shared_account_count': _encode_parameter(shared_account_count),
            'shared_redemption_url': _encode_parameter(shared_redemption_url),
            'collect_shipping_address': _encode_parameter(collect_shipping_address),
            'collect_address': _encode_parameter(collect_address),
            'delivery_zone': _encode_parameter(delivery_zone),
            'default_country': _encode_parameter(default_country)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Term)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.deployment_app_details import DeploymentAppDetails
from pianosdk.publisher.models.user import User
from typing import List


class DeploymentDetails(BaseModel):
    deployment_id: Optional[str] = None
    status: Optional[str] = None
    offer_template_id: Optional[str] = None
    offer_template_name: Optional[str] = None
    description: Optional[str] = None
    version: Optional[int] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    applications: Optional['List[DeploymentAppDetails]'] = None


DeploymentDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class BulkUserImport(BaseModel):
    bulk_user_import_id: Optional[str] = None
    bulk_user_import_created: Optional[datetime] = None
    bulk_user_import_completed: Optional[datetime] = None


BulkUserImport.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.change_subscription_term_step import ChangeSubscriptionTermStep
from pianosdk.publisher.models.send_email_step import SendEmailStep
from pianosdk.publisher.models.upgrade_subscription_step import UpgradeSubscriptionStep


class Action(BaseModel):
    action_id: Optional[str] = None
    type: Optional[str] = None
    number_of_items: Optional[int] = None
    status: Optional[str] = None
    schedule_time: Optional[datetime] = None
    change_subscription_term_step: Optional['ChangeSubscriptionTermStep'] = None
    readonly: Optional[bool] = None
    send_email_step: Optional['SendEmailStep'] = None
    upgrade_subscription_step: Optional['UpgradeSubscriptionStep'] = None


Action.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class App(BaseModel):
    aid: Optional[str] = None
    default_lang: Optional[str] = None
    email_lang: Optional[str] = None
    details: Optional[str] = None
    email: Optional[str] = None
    name: Optional[str] = None
    user_provider: Optional[str] = None
    url: Optional[str] = None
    logo1: Optional[str] = None
    logo2: Optional[str] = None
    state: Optional[str] = None
    private_key: Optional[str] = None
    api_token: Optional[str] = None


App.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class DynamicSubscriptionRenewalConfirmationDto(BaseModel):
    renewal_billing_plan: Optional[str] = None
    renewal_price: Optional[str] = None


DynamicSubscriptionRenewalConfirmationDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.country_model import CountryModel


class AvalaraOriginAddress(BaseModel):
    address_line1: Optional[str] = None
    country: Optional['CountryModel'] = None
    city: Optional[str] = None
    postal_code: Optional[str] = None
    state: Optional[str] = None
    reference_field: Optional[str] = None


AvalaraOriginAddress.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class NewscycleConfigurationTestResult(BaseModel):
    is_url_valid: Optional[bool] = None
    is_api_url_valid: Optional[bool] = None
    is_web_pages_url_valid: Optional[bool] = None
    is_site_id_valid: Optional[bool] = None


NewscycleConfigurationTestResult.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.create_access_period_params import CreateAccessPeriodParams
from typing import List


class CreateBillingConfigurationRequest(BaseModel):
    periods: Optional['List[CreateAccessPeriodParams]'] = None


CreateBillingConfigurationRequest.model_rebuild()



from typing import Optional, Dict, Type, TypeVar, List

from pydantic import BaseModel, Field


class Event(BaseModel):
    version: int
    event: str
    type: str


class AccessGrantedEvent(Event):
    aid: Optional[str]
    expires: Optional[int]
    term_id: Optional[str]
    uid: Optional[str]
    rid: Optional[str]
    access_id: Optional[str]
    user_email: Optional[str]
    contract_id: Optional[str]
    payment_id: Optional[str]
    conversion_id: Optional[str]


class AccessModifiedEvent(Event):
    aid: Optional[str]
    expires: Optional[int]
    term_id: Optional[str]
    uid: Optional[str]
    rid: Optional[str]
    access_id: Optional[str]
    user_email: Optional[str]
    contract_id: Optional[str]
    payment_id: Optional[str]
    conversion_id: Optional[str]


class AccessRevokedEvent(Event):
    aid: Optional[str]
    expires: Optional[int]
    term_id: Optional[str]
    uid: Optional[str]
    rid: Optional[str]
    access_id: Optional[str]
    user_email: Optional[str]
    contract_id: Optional[str]


class AddressUpdatedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    address_id: Optional[str]
    subscription_id: Optional[List[str]]
    voucher_id: Optional[List[str]]


class ContractCreatedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]


class ContractDeletedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]


class ContractRedeemedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    conversion_id: Optional[str]
    uid: Optional[str]
    user_email: Optional[str]
    expires: Optional[int]
    access_id: Optional[str]


class ContractRenewedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    conversion_id: Optional[str]
    uid: Optional[str]
    user_email: Optional[str]
    expires: Optional[int]
    access_id: Optional[str]


class ContractUpdatedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]


class ContractUserAccessExpiredEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    uid: Optional[str]
    user_email: Optional[str]


class ContractUserAccessRevokedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    uid: Optional[str]
    user_email: Optional[str]


class ContractUserCreatedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    user_email: Optional[str]


class ContractUserInvitedEvent(Event):
    aid: Optional[str]
    contract_id: Optional[str]
    rid: Optional[str]
    term_id: Optional[str]
    user_email: Optional[str]


class KeyingEvent(Event):
    aid: Optional[str]
    timestamp: Optional[int]
    content_id: Optional[str]


class LicenseeCreatedEvent(Event):
    aid: Optional[str]
    licensee_id: Optional[str]


class LicenseeUpdatedEvent(Event):
    aid: Optional[str]
    licensee_id: Optional[str]


class PaymentRefundEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    payment_id: Optional[str]
    amount: Optional[float]
    subscription_id: Optional[str]
    access_id: Optional[str]


class SharedSubscriptionChildAccessGrantedEvent(Event):
    aid: Optional[str]
    expires: Optional[int]
    email: Optional[str]
    term_id: Optional[str]
    uid: Optional[str]
    rid: Optional[str]
    access_id: Optional[str]
    user_email: Optional[str]
    contract_id: Optional[str]
    payment_id: Optional[str]
    conversion_id: Optional[str]
    parent_uid: Optional[str]
    parent_subscription_id: Optional[str]


class SubscriptionAutoRenewChangedByEndUserEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    subscription_id: Optional[str]
    term_id: Optional[str]
    auto_renew: Optional[str]


class SubscriptionRenewalEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    rid: Optional[str]
    subscription_id: Optional[str]
    status: Optional[str]
    billing_plan: Optional[str]
    start_date: Optional[int]
    next_bill_date: Optional[int]
    term_id: Optional[str]
    term_type: Optional[str]
    access_id: Optional[str]
    user_email: Optional[str]
    upi_id: Optional[str]
    auto_renew: Optional[bool]
    is_in_grace: Optional[bool]
    grace_period_start_date: Optional[int]
    grace_period_length: Optional[int]
    failure_counter: Optional[int]
    passive_churn_logic_id: Optional[str]
    decline_reason: Optional[str]
    payment_id: Optional[str]
    renewed_by: Optional[str]
    renewal_type: Optional[str]


class TermChangeEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    user_email: Optional[str]
    previous_term_id: Optional[str]
    previous_term_name: Optional[str]
    new_term_id: Optional[str]
    new_term_name: Optional[str]
    previous_subscription_id: Optional[str]
    new_subscription_id: Optional[str]
    new_subscription_rid: Optional[str]
    new_subscription_access_id: Optional[str]
    new_billing_plan: Optional[str]
    new_subscription_next_bill_date: Optional[int]
    date_of_access_change: Optional[int]
    date_of_billing_change: Optional[int]
    upi_id: Optional[str]
    is_in_grace: Optional[bool]
    grace_period_start_date: Optional[int]
    grace_period_length: Optional[int]
    failure_counter: Optional[int]
    passive_churn_logic_id: Optional[str]
    decline_reason: Optional[str]
    payment_id: Optional[str]
    changed_by: Optional[str]


class TermChangeFinishedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    previous_term_id: Optional[str]
    previous_term_name: Optional[str]
    new_term_id: Optional[str]
    new_term_name: Optional[str]
    previous_subscription_id: Optional[str]
    new_subscription_id: Optional[str]
    date_of_access_change: Optional[int]
    date_of_billing_change: Optional[int]
    new_billing_plan: Optional[str]


class TermChangedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    previous_term_id: Optional[str]
    previous_term_name: Optional[str]
    new_term_id: Optional[str]
    new_term_name: Optional[str]
    previous_subscription_id: Optional[str]
    new_subscription_id: Optional[str]
    date_of_access_change: Optional[int]
    date_of_billing_change: Optional[int]
    new_billing_plan: Optional[str]


class TestEvent(Event):
    aid: Optional[str]
    timestamp: Optional[int]


class UnknownEvent(Event):
    pass


class UnsupportedEvent(Event):
    pass


class UserAddressUpdatedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    term_id: Optional[str]
    sub_id: Optional[str]
    subscription_id: Optional[str]
    access_id: Optional[str]
    psc_subscriber_number: Optional[str]


class UserCreatedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    timestamp: Optional[str]


class UserDisabledEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    timestamp: Optional[str]


class UserEmailConfirmedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    timestamp: Optional[str]


class UserPaymentMethodEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    user_payment_info_id: Optional[str]
    user_subscriptions: Optional[List[str]]


class UserUpdatedEvent(Event):
    aid: Optional[str]
    uid: Optional[str]
    timestamp: Optional[str]
    updated_custom_fields: Optional[str]


class VoucherDeliveredEvent(Event):
    aid: Optional[str]
    voucher_id: Optional[str]
    voucher_state: Optional[int]
    voucher_send_date_timestamp: Optional[int]
    voucher_recipient_name: Optional[str]
    voucher_recipient_email: Optional[str]
    voucher_message: Optional[str]
    voucher_code: Optional[str]
    voucher_expires_timestamp: Optional[int]
    voucher_issue_term_conversion_id: Optional[str]
    voucher_redeem_term_conversion_id: Optional[str]
    term_id: Optional[str]
    address_id: Optional[str]


class VoucherPurchasedEvent(Event):
    aid: Optional[str]
    voucher_id: Optional[str]
    voucher_state: Optional[int]
    voucher_send_date_timestamp: Optional[int]
    voucher_recipient_name: Optional[str]
    voucher_recipient_email: Optional[str]
    voucher_message: Optional[str]
    voucher_code: Optional[str]
    voucher_expires_timestamp: Optional[int]
    voucher_issue_term_conversion_id: Optional[str]
    voucher_redeem_term_conversion_id: Optional[str]
    term_id: Optional[str]
    address_id: Optional[str]


class VoucherRedeemedEvent(Event):
    aid: Optional[str]
    voucher_id: Optional[str]
    voucher_state: Optional[int]
    voucher_send_date_timestamp: Optional[int]
    voucher_recipient_name: Optional[str]
    voucher_recipient_email: Optional[str]
    voucher_message: Optional[str]
    voucher_code: Optional[str]
    voucher_expires_timestamp: Optional[int]
    voucher_issue_term_conversion_id: Optional[str]
    voucher_redeem_term_conversion_id: Optional[str]
    term_id: Optional[str]
    address_id: Optional[str]


class VoucherRevokedEvent(Event):
    aid: Optional[str]
    voucher_id: Optional[str]
    voucher_state: Optional[int]
    voucher_send_date_timestamp: Optional[int]
    voucher_recipient_name: Optional[str]
    voucher_recipient_email: Optional[str]
    voucher_message: Optional[str]
    voucher_code: Optional[str]
    voucher_expires_timestamp: Optional[int]
    voucher_issue_term_conversion_id: Optional[str]
    voucher_redeem_term_conversion_id: Optional[str]
    term_id: Optional[str]
    address_id: Optional[str]


_E = TypeVar('_E', bound=Event)
_event_types_mapping: Dict[str, Type[_E]] = {
    'unknown': UnknownEvent, 
    'test': TestEvent, 
    'user_payment_method': UserPaymentMethodEvent, 
    'unsupported': UnsupportedEvent, 
    'payment_refund': PaymentRefundEvent, 
    'address_updated': AddressUpdatedEvent, 
    'term_change': TermChangeEvent, 
    'content_algorithm': KeyingEvent, 
    'term_changed': TermChangedEvent, 
    'subscription_auto_renew_changed': SubscriptionAutoRenewChangedByEndUserEvent, 
    'user_address_updated': UserAddressUpdatedEvent, 
    'voucher_redeemed': VoucherRedeemedEvent, 
    'voucher_delivered': VoucherDeliveredEvent, 
    'voucher_revoked': VoucherRevokedEvent, 
    'voucher_purchased': VoucherPurchasedEvent, 
    'user_updated': UserUpdatedEvent, 
    'user_email_confirmed': UserEmailConfirmedEvent, 
    'user_created': UserCreatedEvent, 
    'user_disabled': UserDisabledEvent, 
    'contract_updated': ContractUpdatedEvent, 
    'contract_created': ContractCreatedEvent, 
    'contract_user_created': ContractUserCreatedEvent, 
    'licensee_invite_to_contract': ContractUserInvitedEvent, 
    'contract_deleted': ContractDeletedEvent, 
    'subscription_renewal': SubscriptionRenewalEvent, 
    'access_granted': AccessGrantedEvent, 
    'access_revoked': AccessRevokedEvent, 
    'access_modified': AccessModifiedEvent, 
    'licensee_created': LicenseeCreatedEvent, 
    'licensee_updated': LicenseeUpdatedEvent, 
    'term_change_finished': TermChangeFinishedEvent, 
    'contract_user_access_expired': ContractUserAccessExpiredEvent, 
    'contract_user_access_revoked': ContractUserAccessRevokedEvent, 
    'contract_redeemed': ContractRedeemedEvent, 
    'contract_renewed': ContractRenewedEvent, 
    'shared_subscription_child': SharedSubscriptionChildAccessGrantedEvent
}



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from typing import Any
from pianosdk.publisher.models.user import User


class PublisherUserApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create(self, aid: str, uid: str, email: str, first_name: str = None, last_name: str = None, create_from_external: bool = None) -> ApiResponse[User]:
        _url_path = '/api/v3/publisher/user/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'email': _encode_parameter(email),
            'first_name': _encode_parameter(first_name),
            'last_name': _encode_parameter(last_name),
            'create_from_external': _encode_parameter(create_from_external)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result

    def disable(self, aid: str, uid: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/user/disable'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def get(self, aid: str, uid: str, disabled: bool = False) -> ApiResponse[User]:
        _url_path = '/api/v3/publisher/user/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'disabled': _encode_parameter(disabled)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result

    def list(self, aid: str, offset: int = 0, limit: int = 100, disabled: bool = False, q: str = None, search_after: str = None) -> ApiResponse[List[User]]:
        _url_path = '/api/v3/publisher/user/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'disabled': _encode_parameter(disabled),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'search_after': _encode_parameter(search_after)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result

    def register(self, aid: str, email: str, first_name: str = None, last_name: str = None) -> ApiResponse[User]:
        _url_path = '/api/v3/publisher/user/register'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'email': _encode_parameter(email),
            'first_name': _encode_parameter(first_name),
            'last_name': _encode_parameter(last_name)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result

    def search(self, aid: str, offset: int = 0, limit: int = 100, uid: str = None, exclude_cf_metadata: bool = False, name: str = None, email: str = None, registered_from: datetime = None, registered_until: datetime = None, access_to_resources: List[str] = None, converted_terms: List[str] = None, access_from: datetime = None, access_until: datetime = None, converted_term_from: datetime = None, converted_term_until: datetime = None, converted_term_sharing_type: str = None, redeemed_promotions: List[str] = None, redeemed_promotion_from: datetime = None, redeemed_promotion_until: datetime = None, trial_period_is_active: bool = None, has_trial_period: bool = None, has_access: bool = None, has_conversion_term: bool = None, has_redeemed_promotion: bool = None, include_trial_redemptions: bool = None, converted_term_types: List[str] = None, has_conversion_term_type: bool = None, spent_money_currency: str = None, spent_money_from: float = None, spent_money_until: float = None, spent_from_date: datetime = None, spent_until_date: datetime = None, payment_methods: List[int] = None, billing_failure_from: datetime = None, billing_failure_until: datetime = None, had_billing_failure: bool = None, has_payment: bool = None, upi_ext_customer_id: str = None, credit_card_will_expire: str = None, active_subscription_to_resources: List[str] = None, has_active_subscription: bool = None, subscription_start_from: datetime = None, subscription_start_until: datetime = None, subscription_renew_from: datetime = None, subscription_renew_until: datetime = None, subscription_expire_from: datetime = None, subscription_expire_until: datetime = None, trial_expire_from: datetime = None, trial_expire_until: datetime = None, has_any_subscriptions: bool = None, has_subscription_will_renew: bool = None, has_subscription_will_expire: bool = None, has_subscription_starts: bool = None, has_unresolved_inquiry: bool = None, submitted_inquiry_from: datetime = None, submitted_inquiry_until: datetime = None, received_response_from: datetime = None, received_response_until: datetime = None, resolved_inquiry_from: datetime = None, resolved_inquiry_until: datetime = None, has_submitted_inquiry: bool = None, has_received_response_inquiry: bool = None, has_resolved_inquiry: bool = None, has_licensing_contract_redemptions: bool = None, selected_licensees: List[str] = None, selected_contracts: List[str] = None, licensing_contract_redeemed_from: datetime = None, licensing_contract_redeemed_until: datetime = None, data_type: List[str] = None, data: str = None, has_data: bool = None, has_last_access_time: bool = None, last_access_time_from: datetime = None, last_access_time_until: datetime = None, selected_consents_map: List[str] = None, consent_checked: bool = None, custom_fields: str = None, source: str = None, invert_credit_card_will_expire: bool = None, has_email_confirmation_required: bool = None, consent_has_data: bool = None, order_by: str = None, order_direction: str = None, q: str = None, search_after: str = None, esdebug: bool = False) -> ApiResponse[List[User]]:
        _url_path = '/api/v3/publisher/user/search'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'exclude_cf_metadata': _encode_parameter(exclude_cf_metadata),
            'name': _encode_parameter(name),
            'email': _encode_parameter(email),
            'registered_from': _encode_parameter(registered_from),
            'registered_until': _encode_parameter(registered_until),
            'access_to_resources': _encode_parameter(access_to_resources),
            'converted_terms': _encode_parameter(converted_terms),
            'access_from': _encode_parameter(access_from),
            'access_until': _encode_parameter(access_until),
            'converted_term_from': _encode_parameter(converted_term_from),
            'converted_term_until': _encode_parameter(converted_term_until),
            'converted_term_sharing_type': _encode_parameter(converted_term_sharing_type),
            'redeemed_promotions': _encode_parameter(redeemed_promotions),
            'redeemed_promotion_from': _encode_parameter(redeemed_promotion_from),
            'redeemed_promotion_until': _encode_parameter(redeemed_promotion_until),
            'trial_period_is_active': _encode_parameter(trial_period_is_active),
            'has_trial_period': _encode_parameter(has_trial_period),
            'has_access': _encode_parameter(has_access),
            'has_conversion_term': _encode_parameter(has_conversion_term),
            'has_redeemed_promotion': _encode_parameter(has_redeemed_promotion),
            'include_trial_redemptions': _encode_parameter(include_trial_redemptions),
            'converted_term_types': _encode_parameter(converted_term_types),
            'has_conversion_term_type': _encode_parameter(has_conversion_term_type),
            'spent_money_currency': _encode_parameter(spent_money_currency),
            'spent_money_from': _encode_parameter(spent_money_from),
            'spent_money_until': _encode_parameter(spent_money_until),
            'spent_from_date': _encode_parameter(spent_from_date),
            'spent_until_date': _encode_parameter(spent_until_date),
            'payment_methods': _encode_parameter(payment_methods),
            'billing_failure_from': _encode_parameter(billing_failure_from),
            'billing_failure_until': _encode_parameter(billing_failure_until),
            'had_billing_failure': _encode_parameter(had_billing_failure),
            'has_payment': _encode_parameter(has_payment),
            'upi_ext_customer_id': _encode_parameter(upi_ext_customer_id),
            'credit_card_will_expire': _encode_parameter(credit_card_will_expire),
            'active_subscription_to_resources': _encode_parameter(active_subscription_to_resources),
            'has_active_subscription': _encode_parameter(has_active_subscription),
            'subscription_start_from': _encode_parameter(subscription_start_from),
            'subscription_start_until': _encode_parameter(subscription_start_until),
            'subscription_renew_from': _encode_parameter(subscription_renew_from),
            'subscription_renew_until': _encode_parameter(subscription_renew_until),
            'subscription_expire_from': _encode_parameter(subscription_expire_from),
            'subscription_expire_until': _encode_parameter(subscription_expire_until),
            'trial_expire_from': _encode_parameter(trial_expire_from),
            'trial_expire_until': _encode_parameter(trial_expire_until),
            'has_any_subscriptions': _encode_parameter(has_any_subscriptions),
            'has_subscription_will_renew': _encode_parameter(has_subscription_will_renew),
            'has_subscription_will_expire': _encode_parameter(has_subscription_will_expire),
            'has_subscription_starts': _encode_parameter(has_subscription_starts),
            'has_unresolved_inquiry': _encode_parameter(has_unresolved_inquiry),
            'submitted_inquiry_from': _encode_parameter(submitted_inquiry_from),
            'submitted_inquiry_until': _encode_parameter(submitted_inquiry_until),
            'received_response_from': _encode_parameter(received_response_from),
            'received_response_until': _encode_parameter(received_response_until),
            'resolved_inquiry_from': _encode_parameter(resolved_inquiry_from),
            'resolved_inquiry_until': _encode_parameter(resolved_inquiry_until),
            'has_submitted_inquiry': _encode_parameter(has_submitted_inquiry),
            'has_received_response_inquiry': _encode_parameter(has_received_response_inquiry),
            'has_resolved_inquiry': _encode_parameter(has_resolved_inquiry),
            'has_licensing_contract_redemptions': _encode_parameter(has_licensing_contract_redemptions),
            'selected_licensees': _encode_parameter(selected_licensees),
            'selected_contracts': _encode_parameter(selected_contracts),
            'licensing_contract_redeemed_from': _encode_parameter(licensing_contract_redeemed_from),
            'licensing_contract_redeemed_until': _encode_parameter(licensing_contract_redeemed_until),
            'data_type': _encode_parameter(data_type),
            'data': _encode_parameter(data),
            'has_data': _encode_parameter(has_data),
            'has_last_access_time': _encode_parameter(has_last_access_time),
            'last_access_time_from': _encode_parameter(last_access_time_from),
            'last_access_time_until': _encode_parameter(last_access_time_until),
            'selected_consents_map': _encode_parameter(selected_consents_map),
            'consent_checked': _encode_parameter(consent_checked),
            'custom_fields': _encode_parameter(custom_fields),
            'source': _encode_parameter(source),
            'invert_credit_card_will_expire': _encode_parameter(invert_credit_card_will_expire),
            'has_emailConfirmation_required': _encode_parameter(has_email_confirmation_required),
            'consent_has_data': _encode_parameter(consent_has_data),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'search_after': _encode_parameter(search_after),
            'esdebug': _encode_parameter(esdebug)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result

    def update(self, aid: str, uid: str, email: str = None, first_name: str = None, last_name: str = None, custom_fields: Any = None) -> ApiResponse[User]:
        _url_path = '/api/v3/publisher/user/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'email': _encode_parameter(email),
            'first_name': _encode_parameter(first_name),
            'last_name': _encode_parameter(last_name)
        }

        _body = custom_fields and custom_fields.dict()
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, User)
        return _result




from pianosdk.user.api.user_api import UserApi
from pianosdk.user.api.user_access_api import UserAccessApi

from pianosdk.user.models.access import Access
from pianosdk.user.models.access_dto import AccessDTO
from pianosdk.user.models.country import Country
from pianosdk.user.models.geo_location import GeoLocation
from pianosdk.user.models.payment_method import PaymentMethod
from pianosdk.user.models.region import Region
from pianosdk.user.models.resource import Resource
from pianosdk.user.models.resource_dto import ResourceDto
from pianosdk.user.models.user import User
from pianosdk.user.models.user_address import UserAddress
from pianosdk.user.models.user_dto import UserDto




import gzip
import json
import re
import urllib.parse
from datetime import datetime
from http.cookies import SimpleCookie
from typing import List, Dict, Callable, Tuple, Optional

from pydantic import Field, BaseModel

from pianosdk.configuration import Configuration
from pianosdk.utils import _decrypt, _unbox_object

_token_pattern = re.compile(r'{(j[azo]x)}([\w-]+)')


class AccessToken(BaseModel):
    rid: str
    expiration_seconds: int = Field(0, alias='ex')
    early_expiration_seconds: int = Field(0, alias='eex')
    access_id: Optional[str] = Field(None, alias='id')
    build_time: int = Field(0, alias='bt')
    created_time: int = Field(0, alias='ct')
    ips: Optional[str]
    uid: Optional[str]

    @property
    def is_expired(self) -> bool:
        seconds = self.early_expiration_seconds or self.expiration_seconds
        return 0 < seconds < datetime.utcnow().timestamp()

    @property
    def is_access_granted(self) -> bool:
        return self.expiration_seconds >= 0 and not self.is_expired


class AccessTokenStorage:
    _tokens: Dict[str, AccessToken] = {}

    def __init__(self, config: Configuration) -> None:
        self.config = config
        self._decoders: Dict[str, Callable[[str], str]] = {
            'a': lambda x: _decrypt(self.config.private_key, x),
            'o': lambda x: x,
            'z': lambda x: gzip.decompress(_decrypt(self.config.private_key, x).encode()).decode(),
        }

    def get_access_token(self, rid: str) -> Optional[AccessToken]:
        return self._tokens.get(rid, AccessToken(rid=rid, ex=-1))

    def parse_access_tokens(self, cookie: str) -> None:
        tokens = _token_pattern.findall(urllib.parse.unquote(cookie))
        self._tokens = {item.rid: item for enc_token in tokens for item in self._decode_token(enc_token)}

    def parse_access_tokens_from_cookies_string(self, cookies: str, cookie_name: str = '__tac') -> None:
        parsed_cookies = SimpleCookie()
        parsed_cookies.load(cookies)
        if cookie_name in parsed_cookies:
            self.parse_access_tokens(parsed_cookies.get(cookie_name).value or '')

    def _decode_token(self, groups: Tuple[str, str]) -> List[AccessToken]:
        encoding, encoded_value = groups
        decode = self._decoders.get(encoding[1])
        if not decode:
            raise ValueError(f'Unknown format {encoding}')
        json_data = json.loads(decode(encoded_value))
        return [item for sublist in [_unbox_object(json_data, AccessToken)] for item in sublist]



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.licensee_notification_rule import LicenseeNotificationRule


class PublisherLicensingNotificationRuleApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def archive_licensee_notification_rule(self, aid: str, licensee_id: str, notification_rule_id: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/licensing/notificationRule/archive'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id),
            'notification_rule_id': _encode_parameter(notification_rule_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def create_licensee_notification_rule(self, aid: str, licensee_id: str, is_for_all_contracts: bool, parameter: str, condition: str, contract_id_list: str = None, condition_value: int = None) -> ApiResponse[LicenseeNotificationRule]:
        _url_path = '/api/v3/publisher/licensing/notificationRule/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id),
            'contract_id_list': _encode_parameter(contract_id_list),
            'is_for_all_contracts': _encode_parameter(is_for_all_contracts),
            'parameter': _encode_parameter(parameter),
            'condition': _encode_parameter(condition),
            'condition_value': _encode_parameter(condition_value)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, LicenseeNotificationRule)
        return _result

    def get_notification_rules_list(self, aid: str, licensee_id: str) -> ApiResponse[List[LicenseeNotificationRule]]:
        _url_path = '/api/v3/publisher/licensing/notificationRule/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, LicenseeNotificationRule)
        return _result

    def update_licensee_notification_rule(self, aid: str, licensee_id: str, notification_rule_id: str, is_for_all_contracts: bool, parameter: str, condition: str, contract_id_list: str = None, condition_value: int = None) -> ApiResponse[LicenseeNotificationRule]:
        _url_path = '/api/v3/publisher/licensing/notificationRule/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id),
            'contract_id_list': _encode_parameter(contract_id_list),
            'notification_rule_id': _encode_parameter(notification_rule_id),
            'is_for_all_contracts': _encode_parameter(is_for_all_contracts),
            'parameter': _encode_parameter(parameter),
            'condition': _encode_parameter(condition),
            'condition_value': _encode_parameter(condition_value)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, LicenseeNotificationRule)
        return _result




from pianosdk.anon.api.access_api import AccessApi
from pianosdk.anon.api.access_token_api import AccessTokenApi
from pianosdk.anon.api.anon_assets_api import AnonAssetsApi
from pianosdk.anon.api.anon_country_list_api import AnonCountryListApi
from pianosdk.anon.api.anon_error_api import AnonErrorApi
from pianosdk.anon.api.anon_mobile_sdk_id_deployment_api import AnonMobileSdkIdDeploymentApi
from pianosdk.anon.api.anon_user_api import AnonUserApi
from pianosdk.anon.api.anon_user_disable_api import AnonUserDisableApi
from pianosdk.anon.api.conversion_api import ConversionApi
from pianosdk.anon.api.conversion_external_api import ConversionExternalApi
from pianosdk.anon.api.conversion_registration_api import ConversionRegistrationApi
from pianosdk.anon.api.email_confirmation_api import EmailConfirmationApi
from pianosdk.anon.api.oauth_api import OauthApi
from pianosdk.anon.api.subscription_api import SubscriptionApi
from pianosdk.anon.api.swg_sync_api import SwgSyncApi



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ImportProfile(BaseModel):
    import_profile_id: Optional[str] = None
    name: Optional[str] = None
    source_environment: Optional[str] = None
    source_aid: Optional[str] = None
    source_url: Optional[str] = None
    source_api_token: Optional[str] = None


ImportProfile.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ConsentBoxEntry(BaseModel):
    field_name: Optional[str] = None
    display_text: Optional[str] = None
    entry: Optional[bool] = None
    create_date: Optional[datetime] = None
    type: Optional[str] = None


ConsentBoxEntry.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TermLink(BaseModel):
    term_id: Optional[str] = None
    period_id: Optional[str] = None
    logic_id: Optional[str] = None
    name: Optional[str] = None
    default: Optional[bool] = None


TermLink.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User


class PromoCode(BaseModel):
    promo_code_id: Optional[str] = None
    promotion_id: Optional[str] = None
    code: Optional[str] = None
    assigned_email: Optional[str] = None
    reserve_date: Optional[datetime] = None
    state: Optional[str] = None
    state_value: Optional[str] = None
    claimed_user: Optional['User'] = None
    claimed_date: Optional[datetime] = None
    create_date: Optional[datetime] = None
    create_by: Optional[str] = None
    update_date: Optional[datetime] = None
    update_by: Optional[str] = None
    deleted: Optional[bool] = None
    last_original_price: Optional[str] = None


PromoCode.model_rebuild()



from dataclasses import dataclass
from typing import TypeVar, Generic, Optional

T = TypeVar('T')


@dataclass
class ApiResponse(Generic[T]):
    request_id: str
    code: int
    ts: int
    data: T
    count: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    total: Optional[int] = None



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class TermConversionData(BaseModel):
    aid: Optional[str] = None
    offer_id: Optional[str] = None
    term_id: Optional[str] = None
    offer_template_id: Optional[str] = None
    template_id: Optional[str] = None
    uid: Optional[str] = None
    user_country: Optional[str] = None
    user_region: Optional[str] = None
    user_city: Optional[str] = None
    zip: Optional[str] = None
    latitude: Optional[str] = None
    longitude: Optional[str] = None
    user_agent: Optional[str] = None
    locale: Optional[str] = None
    hour: Optional[str] = None
    url: Optional[str] = None
    browser: Optional[str] = None
    platform: Optional[str] = None
    operating_system: Optional[str] = None
    tags: Optional[str] = None
    content_created: Optional[str] = None
    content_author: Optional[str] = None
    content_section: Optional[str] = None
    campaigns: Optional[List[str]] = None


TermConversionData.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PublisherLoginRequest(BaseModel):
    aid: Optional[str] = None
    email: Optional[str] = None
    password: Optional[str] = None
    stay_logged_in: Optional[bool] = None
    alias_name: Optional[str] = None


PublisherLoginRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class OneSourceConfiguration(BaseModel):
    onesource_url: Optional[str] = None
    onesource_username: Optional[str] = None
    onesource_password: Optional[str] = None
    onesource_company_name: Optional[str] = None


OneSourceConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User


class LightGlobalTemplate(BaseModel):
    offer_template_id: Optional[str] = None
    aid: Optional[str] = None
    name: Optional[str] = None
    offer_template_name: Optional[str] = None
    description: Optional[str] = None
    create_date: Optional[datetime] = None
    create_by: Optional['User'] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    publish_date: Optional[datetime] = None
    version: Optional[int] = None
    thumbnail_image_url: Optional[str] = None
    live_thumbnail_image_url: Optional[str] = None
    is_published: Optional[bool] = None
    count_variants: Optional[int] = None
    count_content_fields: Optional[int] = None
    deployment_id: Optional[str] = None
    status: Optional[str] = None


LightGlobalTemplate.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CommentAction(BaseModel):
    id: Optional[str] = None
    caption: Optional[str] = None


CommentAction.model_rebuild()
